// Arduino Mega code for Instructables Zero Gravity Grow Box

// #include <Wire.h>
#include <EEPROM.h>  // save ctl values for power off
#include "DHT.h"

byte growboxID = 1;  //shd devise smarter way to vary this

#define DHTPin A0     // thermometer/humidity pin
//#define DHTTYPE DHT11   // DHT 11
#define DHTTYPE DHT21   // DHT 21
DHT dhtB(DHTPin, DHTTYPE);
float tempB = 20;
float humidB = 20;
byte currentTemp;
byte currentHumid;
unsigned long lastTemp;

#define rotorPin 10  //relay control of DC motor
#define airFanPin 11  //relay for fan
byte fanFlag = 2; // automatic. 0=off 1=on
boolean motorOn = true;  //to be set by BT controls

#define blueLitePin 12
byte blueLiteOn = 0; //hours, to be set by BT ctl
byte blueLiteWk = 0;
byte blueLiteStatus = 0; //off
int blueLiteOnEELoc = 1;  //addr in eeprom;

#define redLitePin 13
byte redLiteOn = 0; //hours, to be set by BT ctl
byte redLiteWk = 0;
byte redLiteStatus = 0; //off
int redLiteOnEELoc = 0;  //addr in eeprom;

#define SensorPinA A8 // soil moisture sensor
#define SensorPinB A9 // soil moisture sensor
#define waterPumpPin 9
boolean waterPumpOn = false;
unsigned long waterPumpOnTime = 0;
byte wetEnough = 0;  //% set by user
byte wetPercent = 100;
unsigned long lastSensorReading = millis();  //    
//int sensorLow = 9999;
//int sensorHigh = 0;
float sensorValueA = 0; 
float sensorValueB = 0; 


#define airPumpPin 8
boolean airPumpOn = true;


byte currentHour = 0; // st
int currentHourEELoc = 2;  //addr in eeprom;
unsigned int daysFromStart = 0;
int daysFromStartEELoc = 3;  //addr in eeprom;
unsigned int daysIntoPhase = 0;
int daysIntoPhaseEELoc = 5;  //addr in eeprom;
unsigned long lastMin = millis();
unsigned long lastHour = millis();

unsigned long sendTime = 0;  //when data last sent to phone
byte wkRotation = 1; 
byte phaseReset = 0;
byte startReset = 0;

void setup() {
  Serial.begin(9600);
  Serial1.begin(9600);  //BT connection to box


  
  pinMode(rotorPin, OUTPUT);
  digitalWrite(rotorPin, HIGH); //on
  pinMode(waterPumpPin, OUTPUT);
  digitalWrite(waterPumpPin, LOW); //off
  pinMode(airFanPin, OUTPUT);
  digitalWrite(airFanPin, LOW); //off
  pinMode(airPumpPin, OUTPUT);    
  digitalWrite(airPumpPin, HIGH); //on
  pinMode(blueLitePin, OUTPUT);    
  digitalWrite(blueLitePin, HIGH); //on
  pinMode(redLitePin, OUTPUT);    
  digitalWrite(redLitePin, HIGH); //on
  pinMode(SensorPinA, INPUT);   //soil moisture 
  pinMode(SensorPinB, INPUT);   //soil moisture 

  // bring in saved variables from non-volatile
  byte wkByte = EEPROM.read(redLiteOnEELoc);
  if (wkByte != 255) {  // stuff has been saved
    redLiteOn = wkByte;
    blueLiteOn = EEPROM.read(blueLiteOnEELoc);
    currentHour = EEPROM.read(currentHourEELoc);
    daysIntoPhase = EEPROMReadInt(daysIntoPhaseEELoc);
    daysFromStart = EEPROMReadInt(daysFromStartEELoc);
//    Serial.print(" eeredLiteOn=");Serial.print(redLiteOn);
//    Serial.print(" eeblueLiteOn=");Serial.print(blueLiteOn);
//    Serial.print(" eecurrentHour=");Serial.print(currentHour);
//    Serial.print(" eedaysFromStart=");Serial.print(daysFromStart);
//    Serial.print(" eedaysIntoPhase=");Serial.println(daysIntoPhase);
  } else {  //write default values  
    EEPROM.update(redLiteOnEELoc, redLiteOn);
    EEPROM.update(blueLiteOnEELoc, blueLiteOn);
    EEPROM.update(currentHourEELoc, currentHour);
    EEPROMWriteInt(daysFromStartEELoc, daysFromStart);
    EEPROMWriteInt(daysIntoPhaseEELoc, daysIntoPhase);
    Serial.println("defaults written to Eprom");
  }

  float workVal = dhtB.readTemperature(true);  //get box temp in F (false gives C)
  if (!isnan(workVal)) tempB = workVal;
  delay(2000);
  workVal = dhtB.readHumidity();  //get box humidity
  if (!isnan(workVal)) humidB = workVal;
  delay(2000);
  lastTemp = millis();  
  
  writeDataToPhone();

  checkLites();  //see if lites need on or off

}

void loop() {

  int reading = checkMoisture();
  checkWetEnuf(reading);  //see if system needs more water
  
    //  check to see if hour elapsed
  if (millis() - lastHour >= 3600000) {
    lastHour = millis();
        // advance the hour 
    currentHour += 1;
    if (currentHour > 23) { // rolled over into new day
      currentHour = 0;
      daysFromStart ++; // update days counter
      daysIntoPhase ++; 
      EEPROMWriteInt(daysFromStartEELoc, daysFromStart);  //needed towrite ints
      EEPROMWriteInt(daysIntoPhaseEELoc, daysIntoPhase);
    }
        // save new hour to eeprom
    EEPROM.update(redLiteOnEELoc, redLiteOn);  //it only writes if changed
    EEPROM.update(blueLiteOnEELoc, blueLiteOn);  
    EEPROM.update(currentHourEELoc, currentHour);
    checkLites();  //see if lites need on or off
  }


   // Read temperatures as Farenheit
  if (millis() - lastTemp > 500) {  // only read temp each half second  
    lastTemp = millis();
    float workVal = dhtB.readTemperature(true);  //get box temp in F (false gives C)
//    Serial.print(" workVal=");Serial.print(workVal);
    if (!isnan(workVal)) {
      tempB = tempB * .6 + workVal * .4; // smooth it in
    }
    currentTemp = tempB;   
    workVal = dhtB.readHumidity();  //get box humidity
//    Serial.print(" workVal=");Serial.println(workVal);
    if (!isnan(workVal)) humidB = workVal;
    currentHumid = humidB;
  }

     // check user setting for fan
   if (fanFlag == 2) {  //automatic  
     // check air temp to control fan
     if (currentTemp > 80 )  { //turn on fan
       digitalWrite(airFanPin, HIGH); //on
     } else if (currentTemp < 75) {
       digitalWrite(airFanPin, LOW); //off
     }
   } else if (fanFlag == 1) digitalWrite(airFanPin, HIGH); //on
   else digitalWrite(airFanPin, LOW); //off

  // exchange data with phone every 2 seconds
    if (millis() - sendTime > 2000) {
      writeDataToPhone();
      sendTime = millis();
      readDataFromPhone(); // see if any data from phone 
      if (wkRotation == 1) {
        digitalWrite(rotorPin, HIGH); //on
      } else {
        digitalWrite(rotorPin, LOW);  //turn off rotor
      }
      if (phaseReset == 1) {
        daysIntoPhase = 0;
        phaseReset = 0;
        EEPROMWriteInt(daysIntoPhaseEELoc, daysIntoPhase);
      }
      if (startReset == 1) {
        daysFromStart = 0;
        currentHour = 0; //9/23/19
        startReset = 0;
        EEPROMWriteInt(daysFromStartEELoc, daysFromStart);  //
      }
      if (redLiteWk != redLiteOn) {
        redLiteOn = redLiteWk;  //put in new value
 //?       EEPROM.update(redLiteOnEELoc, redLiteOn);  //maybe not needed??
        checkLites(); // turn on or off
      }
      if (blueLiteWk != blueLiteOn) {
        blueLiteOn = blueLiteWk;  //put in new value
 //?       EEPROM.update(blueLiteOnEELoc, blueLiteOn);  
        checkLites();
      }
    }
}

int checkMoisture() {        // Check soil moisture periodically
  if (millis() - lastSensorReading > 30000) { // every 30 seconds
    sensorValueA = 0; sensorValueB = 0;
    for (int x=0; x<100; x++) { //take 100 quick readings
      sensorValueA = sensorValueA + analogRead(SensorPinA); 
      sensorValueB = sensorValueB + analogRead(SensorPinB); 
    }
    sensorValueA = sensorValueA / 100;
    sensorValueB = sensorValueB / 100;
    int sensorAvg = 0;
      Serial.print(" sensorValueA=");Serial.print(sensorValueA);
      Serial.print(" sensorValueB=");Serial.println(sensorValueB);
    sensorAvg = (sensorValueA+sensorValueB)/2; 
  
    int wkPercent = map(sensorAvg, 0, 650, 0, 100); //calc % 
    if (wkPercent > 0) { //valid
      if (wkPercent > 100) wkPercent = 100;
//      wetPercent = wetPercent * .8 + wkPercent * .2; //smooth it in
      wetPercent = wkPercent; //
      Serial.print(" sensorAvg=");Serial.print(sensorAvg);
      Serial.print(" wetPercent=");Serial.println(wetPercent);
    } else {
      Serial.print(" sensorAvg=");Serial.print(sensorAvg);
      Serial.print(" wkPercent=");Serial.println(wkPercent);
    }
    lastSensorReading = millis();  //start over
  }
  return wetPercent;
}

void checkWetEnuf(int wetPct) {  
  if (wetPct < wetEnough && waterPumpOn == false) {  
      //need to turn on water pump for a few seconds
    digitalWrite(waterPumpPin, HIGH); //
    waterPumpOnTime = millis();  
    waterPumpOn = true;
  } else if (millis() - waterPumpOnTime > 10000) { //try limit 10 seconds on
    digitalWrite(waterPumpPin, LOW); // turn it off
    //now give it a chance 5 min to soak through before allowing it on again
    if (millis() - waterPumpOnTime > 300000) waterPumpOn = false; 
  }
}


void writeDataToPhone() {
  Serial1.write(growboxID);
  Serial1.write(currentHour);
  Serial1.write(redLiteStatus);
  Serial1.write(blueLiteStatus);
  Serial1.write(wetPercent);
  Serial1.write(currentTemp);
  Serial1.write(currentHumid);
  if (daysIntoPhase < 1000) Serial1.print('0');
  if (daysIntoPhase < 100) Serial1.print('0');
  if (daysIntoPhase < 10) Serial1.print('0');
  Serial1.print(daysIntoPhase);
  if (daysFromStart < 1000) Serial1.print('0');
  if (daysFromStart < 100) Serial1.print('0');
  if (daysFromStart < 10) Serial1.print('0');
  Serial1.print(daysFromStart);
  Serial1.println(""); //
 
}

void checkLites()  {  //see if lites need on or off
    if (currentHour >= redLiteOn) {  //time to turn off
      digitalWrite(redLitePin, LOW); //
      redLiteStatus = 0;
    } else {
      digitalWrite(redLitePin, HIGH); //  on
      redLiteStatus = 1;
    }
    if (currentHour >= blueLiteOn) {  //time to turn off
      digitalWrite(blueLitePin, LOW); //
      blueLiteStatus = 0;
    } else {
      digitalWrite(blueLitePin, HIGH); //  on
      blueLiteStatus = 1;
    }
}

void readDataFromPhone() { // bring in message line from phone
  while (Serial1.available()) {
    // get the new byte:
    char inChar = (char)Serial1.read();
    // add it to the inputString:
    if (inChar ==  '#') {  //start of message
      wetEnough = (char)Serial1.read(); 
      wkRotation = (char)Serial1.read(); 
      phaseReset = (char)Serial1.read();
      startReset = (char)Serial1.read();
      fanFlag = (char)Serial1.read();
      redLiteWk = (char)Serial1.read(); 
       blueLiteWk = (char)Serial1.read(); 
     }   else {
      char nothing =(char)Serial1.read();  //read until start of message
    }
 //   Serial.print(wetEnough);Serial.print(",");  
//    Serial.print(wkRotation);Serial.print(",");  
//    Serial.print(phaseReset);Serial.print(",");  
//    Serial.print(startReset);Serial.print(",");  
//    Serial.print(fanFlag);Serial.print(",");  
//    Serial.print(redLiteWk);Serial.print(",");  
//    Serial.print(blueLiteWk);Serial.println(",");  
  }
}

void EEPROMWriteInt(int address, int value)
{
  byte two = (value & 0xFF);
  byte one = ((value >> 8) & 0xFF);
  
  EEPROM.update(address, two);
  EEPROM.update(address + 1, one);
}
 
int EEPROMReadInt(int address)
{
  long two = EEPROM.read(address);
  long one = EEPROM.read(address + 1);
 
  return ((two << 0) & 0xFFFFFF) + ((one << 8) & 0xFFFFFFFF);
}
